--- ices-0.4/src/in_mp4.c	2004-07-31 04:35:29.000000000 +0200
+++ ices-0.4-patched/src/in_mp4.c	2005-06-24 22:33:33.000000000 +0200
@@ -41,6 +41,7 @@
 } mp4_in_t;
 
 /* -- static prototypes -- */
+static void ices_mp4_read_metadata (MP4FileHandle mp4file);
 static int ices_mp4_readpcm (input_stream_t* self, size_t len,
 				int16_t* left, int16_t* right);
 static int ices_mp4_close (input_stream_t* self);
@@ -68,7 +69,9 @@
   if (! self->filesize)
     return 1;
 
-  if ((mp4file = MP4Read(self->path, 0)) == MP4_INVALID_FILE_HANDLE)
+  if ((mp4file = MP4Read(self->path, 0)) != MP4_INVALID_FILE_HANDLE) {
+    ices_mp4_read_metadata(mp4file);
+  } else
     return 1;
 
   /* find audio stream */
@@ -140,6 +143,67 @@
   return -1;
 }
 
+static void 
+ices_mp4_read_metadata (MP4FileHandle mp4file)
+{
+  char *value;
+  char *artist = NULL, *title = NULL;
+  uint16_t numvalue, numvalue2;
+  if (MP4GetMetadataName(mp4file, &value) && value != NULL) {
+    ices_log_debug("Title: %s", value);
+    title = ices_util_strdup(value);
+    ices_util_free(value);
+  }
+  if (MP4GetMetadataArtist(mp4file, &value) && value != NULL) {
+    ices_log_debug("Artist: %s", value);
+    artist = ices_util_strdup(value);
+    ices_util_free(value);
+  }
+  if (MP4GetMetadataWriter(mp4file, &value) && value != NULL) {
+    ices_log_debug("Writer: %s", value);
+    ices_util_free(value);
+  }
+  if (MP4GetMetadataComment(mp4file, &value) && value != NULL) {
+    ices_log_debug("Comment: %s", value);
+    ices_util_free(value);
+  }
+  if (MP4GetMetadataTool(mp4file, &value) && value != NULL) {
+    ices_log_debug("Tool: %s", value);
+    ices_util_free(value);
+  }
+  if (MP4GetMetadataYear(mp4file, &value) && value != NULL) {
+    ices_log_debug("Year: %s", value);
+    ices_util_free(value);
+  }
+  if (MP4GetMetadataAlbum(mp4file, &value) && value != NULL) {
+    ices_log_debug("Album: %s", value);
+    ices_util_free(value);
+  }
+  if (MP4GetMetadataTrack(mp4file, &numvalue, &numvalue2)) {
+    ices_log_debug("Track: %u of %u", numvalue,
+		   numvalue2);
+  }
+  if (MP4GetMetadataDisk(mp4file, &numvalue, &numvalue2)) {
+    ices_log_debug("Disk: %u of %u", numvalue,
+		   numvalue2);
+  }
+  if (MP4GetMetadataGenre(mp4file, &value) && value != NULL) {
+    ices_log_debug("Genre: %s", value);
+    ices_util_free(value);
+  }
+  if (MP4GetMetadataTempo(mp4file, &numvalue)) {
+    ices_log_debug("Tempo: %u", numvalue);
+  }
+  ices_metadata_set(artist, title);
+  if (artist)
+    ices_util_free(artist);
+  if (title)
+    ices_util_free(title);
+
+  return;
+}
+
+
 static int
 ices_mp4_readpcm (input_stream_t* self, size_t olen, int16_t* left,
 		     int16_t* right)